@@ -7,17 +7,20 @@
  *
  */
 
-#include "cs-gdk-event-filter.h"
 #include "config.h"
+#include "cs-gdk-event-filter.h"
+#include "cs-screen.h"
 
 #ifdef HAVE_SHAPE_EXT
 #include <X11/extensions/shape.h>
 #endif
+#include <X11/Xatom.h>
 #include <gtk/gtkx.h>
 #include <string.h>
 
 enum {
         XSCREEN_SIZE,
+        SCREENSAVER_WINDOW_CHANGED,
         LAST_SIGNAL
 };
 
@@ -25,12 +28,6 @@ static guint signals [LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE (CsGdkEventFilter, cs_gdk_event_filter, G_TYPE_OBJECT)
 
-static gboolean
-we_are_fallback_window (CsGdkEventFilter *filter)
-{
-    return filter->pretty_xid > 0;
-}
-
 static gboolean
 ignore_fcitx_input_window (CsGdkEventFilter *filter, Window xid)
 {
@@ -110,43 +107,73 @@ unshape_window (CsGdkEventFilter *filter)
 }
 
 static void
-raise_managed_window (CsGdkEventFilter *filter)
+restack (CsGdkEventFilter *filter,
+         Window            event_window,
+         const gchar      *event_type)
 {
-    GdkWindow *win;
+    g_autofree gchar *net_wm_name = NULL;
 
-    g_return_if_fail (CS_IS_GDK_EVENT_FILTER (filter));
+    gdk_x11_display_error_trap_push (filter->display);
 
-    win = gtk_widget_get_window (GTK_WIDGET (filter->managed_window));
+    net_wm_name = cs_screen_get_net_wm_name (event_window);
 
-    gdk_window_raise (win);
-}
-
-static gboolean
-x11_window_is_ours (CsGdkEventFilter *filter, Window window)
-{
-    GdkWindow *gwindow;
-    gboolean   ret;
-
-    ret = FALSE;
-
-    gwindow = gdk_x11_window_lookup_for_display (filter->display, window);
-    if (gwindow && (window != GDK_ROOT_WINDOW ())) {
-            ret = TRUE;
-    }
-
-    return ret;
-}
-
-static gboolean
-is_pretty_window (CsGdkEventFilter *filter,
-                  Window            window)
-{
-    if (filter->pretty_xid == 0)
+    if (g_strcmp0 (net_wm_name, "event-grabber-window") == 0)
     {
-        return FALSE;
+        g_debug ("(Ignoring %s from CsEventGrabber window)", event_type);
+        gdk_x11_display_error_trap_pop_ignored (filter->display);
+        return;
     }
 
-    return window == filter->pretty_xid;
+    // Screensaver windows get re-made but we want to pick up new ones
+    // so the backup locker can stay in place always.
+    if (filter->pretty_xid != event_window)
+    {
+        if (g_strcmp0 (net_wm_name, "cinnamon-screensaver-window") == 0)
+        {
+            g_debug ("New screensaver window found: 0x%lx (replaces 0x%lx)", event_window, filter->pretty_xid);
+            filter->pretty_xid = event_window;
+            g_signal_emit (filter, signals[SCREENSAVER_WINDOW_CHANGED], 0, event_window);
+        }
+    }
+
+    if (filter->we_are_backup_window)
+    {
+        if (event_window != filter->pretty_xid)
+        {
+            g_debug ("BackupWindow received %s from window '%s' (0x%lx), raising ourselves.",
+                      event_type,
+                      net_wm_name,
+                      event_window);
+
+            XRaiseWindow(GDK_DISPLAY_XDISPLAY (filter->display), filter->my_xid);
+        }
+        else
+        {
+            g_debug ("BackupWindow received %s from screensaver window (0x%lx), restacking us below it.",
+                      event_type,
+                      event_window);
+
+            Window windows[] = {
+                filter->pretty_xid,
+                filter->my_xid
+            };
+
+            XRestackWindows (GDK_DISPLAY_XDISPLAY (filter->display), windows, 2);
+        }
+    }
+    else
+    {
+        g_debug ("Screensaver received %s from window '%s' (0x%lx), raising ourselves.",
+                  event_type,
+                  net_wm_name,
+                  event_window);
+
+        XRaiseWindow(GDK_DISPLAY_XDISPLAY (filter->display), filter->my_xid);
+    }
+
+    XFlush (GDK_DISPLAY_XDISPLAY (filter->display));
+
+    gdk_x11_display_error_trap_pop_ignored (filter->display);
 }
 
 static GdkFilterReturn
@@ -163,45 +190,63 @@ cs_gdk_event_filter_xevent (CsGdkEventFilter *filter,
           {
             XMapEvent *xme = &ev->xmap;
 
-            if (ignore_fcitx_input_window (filter, xme->window) && we_are_fallback_window (filter))
+            if (ignore_fcitx_input_window (filter, xme->window) && filter->we_are_backup_window)
+            {
+                g_debug ("Ignoring MapNotify for fcitx window (we're the backup-locker).");
+                break;
+            }
+
+            // Ignore my own events.
+            if (xme->window == filter->my_xid)
             {
                 break;
             }
 
-            if (! x11_window_is_ours (filter, xme->window) && !is_pretty_window (filter, xme->window)) {
-                raise_managed_window (filter);
-            }
-
+            restack (filter, xme->window, "MapNotify");
             break;
           }
         case ConfigureNotify:
           {
             XConfigureEvent *xce = &ev->xconfigure;
 
-            if (ignore_fcitx_input_window (filter, xce->window) && we_are_fallback_window (filter))
+            // If the reported window is the root window, and we're the backup window (we have a pretty
+            // xid) then signal to resize to the root window (screen).
+            if (xce->window == GDK_ROOT_WINDOW ())
+            {
+                // Screen size may have changed, tell the fallback
+                g_debug ("ConfigureNotify from root window (0x%lx), screen size may have changed. %s",
+                             xce->window,
+                             filter->we_are_backup_window ? "(we're the backup-locker)" : "");
+
+                // The screensaver doesn't need to know, it will get notified by CsScreen.
+                if (filter->we_are_backup_window)
+                {
+                    g_signal_emit (filter, signals[XSCREEN_SIZE], 0);
+                }
+                break;
+            }
+
+            if (ignore_fcitx_input_window (filter, xce->window) && filter->we_are_backup_window)
+            {
+                g_debug ("Ignoring ConfigureNotify for fcitx window (we're the backup-locker).");
+                break;
+            }
+
+            // Ignore my own events
+            if (xce->window == filter->my_xid)
             {
                 break;
             }
 
-            if (! x11_window_is_ours (filter, xce->window) && !is_pretty_window (filter, xce->window)) {
-                raise_managed_window (filter);
-            }
-
-            // If the reported window is the root window, and we're the backup window (we have a pretty
-            // xid) then signal to resize to the root window (screen).
-            // if (xce->window == GDK_ROOT_WINDOW () && filter->pretty_xid > 0)
-            if (xce->window == GDK_ROOT_WINDOW ())
-            {
-                // Screen size may have changed, tell the fallback
-                g_signal_emit (filter, signals[XSCREEN_SIZE], 0);
-            }
-
+            restack (filter, xce->window, "ConfigureNotify");
             break;
           }
         default:
           {
 #ifdef HAVE_SHAPE_EXT
             if (ev->xany.type == (filter->shape_event_base + ShapeNotify)) {
+                g_debug ("ShapeNotify event. %s",
+                             filter->we_are_backup_window ? "(we're the backup-locker)" : "");
                 unshape_window (filter);
             }
 #endif
@@ -249,6 +294,24 @@ select_shape_events (CsGdkEventFilter *filter)
 #endif
 }
 
+static void
+disable_unredirection (CsGdkEventFilter *filter)
+{
+    GdkWindow *gdk_window;
+    guchar _NET_WM_BYPASS_COMPOSITOR_HINT_OFF = 2;
+
+    gdk_window = gtk_widget_get_window (filter->managed_window);
+
+    gdk_x11_display_error_trap_push (filter->display);
+
+    XChangeProperty (GDK_DISPLAY_XDISPLAY (filter->display), GDK_WINDOW_XID (gdk_window),
+                     XInternAtom (GDK_DISPLAY_XDISPLAY (filter->display), "_NET_WM_BYPASS_COMPOSITOR", TRUE),
+                     XA_CARDINAL, 32, PropModeReplace, &_NET_WM_BYPASS_COMPOSITOR_HINT_OFF, 1);
+    XFlush (GDK_DISPLAY_XDISPLAY (filter->display));
+
+    gdk_x11_display_error_trap_pop_ignored (filter->display);
+}
+
 static GdkFilterReturn
 xevent_filter (GdkXEvent *xevent,
                GdkEvent  *event,
@@ -263,6 +326,7 @@ cs_gdk_event_filter_init (CsGdkEventFilter *filter)
     filter->shape_event_base = 0;
     filter->managed_window = NULL;
     filter->pretty_xid = 0;
+    filter->my_xid = 0;
 }
 
 static void
@@ -294,15 +358,58 @@ cs_gdk_event_filter_class_init (CsGdkEventFilterClass *klass)
                                               0,
                                               NULL, NULL, NULL,
                                               G_TYPE_NONE, 0);
+        signals[SCREENSAVER_WINDOW_CHANGED] = g_signal_new ("screensaver-window-changed",
+                                              G_TYPE_FROM_CLASS (object_class),
+                                              G_SIGNAL_RUN_LAST,
+                                              0,
+                                              NULL, NULL, NULL,
+                                              G_TYPE_NONE, 1, G_TYPE_ULONG);
+}
+
+static void
+muted_log_handler (const char     *log_domain,
+                   GLogLevelFlags  log_level,
+                   const char     *message,
+                   gpointer        data)
+{
+  /* Intentionally empty to discard message */
 }
 
 void
-cs_gdk_event_filter_start (CsGdkEventFilter *filter)
+cs_gdk_event_filter_start (CsGdkEventFilter *filter,
+                           gboolean          fractional_scaling,
+                           gboolean          debug)
 {
     select_popup_events (filter);
     select_shape_events (filter);
 
+    if (fractional_scaling)
+    {
+        disable_unredirection (filter);
+    }
+
+    if (debug)
+    {
+        g_log_set_handler ("Cvc", G_LOG_LEVEL_DEBUG,
+                           muted_log_handler, NULL);
+        g_setenv ("G_MESSAGES_DEBUG", "all", TRUE);
+    }
+
+    filter->my_xid = gdk_x11_window_get_xid (gtk_widget_get_window (GTK_WIDGET (filter->managed_window)));
+
+    g_debug ("Starting event filter for %s - 0x%lx", 
+                 filter->we_are_backup_window ? "backup-locker." : "screensaver.",
+                 filter->my_xid);
     gdk_window_add_filter (NULL, (GdkFilterFunc) xevent_filter, filter);
+
+    if (filter->we_are_backup_window)
+    {
+        restack (filter, filter->pretty_xid, NULL);
+    }
+    else
+    {
+        restack (filter, 0, NULL);
+    }
 }
 
 void
@@ -313,7 +420,7 @@ cs_gdk_event_filter_stop (CsGdkEventFilter *filter)
 
 CsGdkEventFilter *
 cs_gdk_event_filter_new (GtkWidget *managed_window,
-                         gulong     pretty_xid)
+                         gulong    pretty_xid)
 {
     CsGdkEventFilter *filter;
 
@@ -324,6 +431,7 @@ cs_gdk_event_filter_new (GtkWidget *managed_window,
     filter->managed_window = g_object_ref (managed_window);
     filter->pretty_xid = pretty_xid;
 
+    filter->we_are_backup_window = filter->pretty_xid != 0;
+
     return filter;
 }
-
