@@ -86,6 +86,9 @@ def process_is_running(name):
     return res != ""
 
 def do_user_switch():
+    GLib.idle_add(do_user_switch_timeout)
+
+def do_user_switch_timeout(data=None):
     if process_is_running("mdm"):
         command = "%s %s" % ("mdmflexiserver", "--startnew Standard")
         ctx = Gdk.Display.get_default().get_app_launch_context()
@@ -134,13 +137,6 @@ def debug_allocation(alloc):
 def CLAMP(value, low, high):
     return max(low, min(value, high))
 
-def have_gtk_version(version_string):
-    [major, minor, micro] = version_string.split(".", 3)
-
-    return Gtk.get_major_version() >= eval(major) and \
-        Gtk.get_minor_version() >= eval(minor) and \
-        Gtk.get_micro_version() >= eval(micro)
-
 def clear_clipboards(widget):
     clipboard = widget.get_clipboard(Gdk.SELECTION_PRIMARY)
     clipboard.clear()
@@ -152,3 +148,7 @@ def clear_clipboards(widget):
 
 def do_quit():
     Gtk.main_quit()
+
+def DEBUG(message):
+    if status.Debug:
+        print(message, flush=True)
\ No newline at end of file
